'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var rollup = _interopDefault(require('rollup'));
var fs = _interopDefault(require('fs'));
var path = _interopDefault(require('path'));
var glob = _interopDefault(require('glob'));
var os = _interopDefault(require('os'));
var resolve = _interopDefault(require('@rollup/plugin-node-resolve'));
var styles = _interopDefault(require('rollup-plugin-styles'));
var rollupPluginTerser = require('rollup-plugin-terser');
var url = _interopDefault(require('@rollup/plugin-url'));
var child_process = require('child_process');
var jsdom = require('jsdom');
var url$1 = _interopDefault(require('url'));

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    if (enumerableOnly) symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    });
    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(Object(source), true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(Object(source)).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

function defaultInputOptions({
  buildDirectory,
  tmpDir
}) {
  return {
    plugins: [resolve({
      browser: true
    }), styles({
      mode: ["extract"],
      modules: true,
      autoModules: id => id.includes(".module.css"),
      sourceMap: true,
      minimize: true
    }), url({
      include: "**/*",
      exclude: "**/*.(js|json|css)",
      destDir: path.resolve(tmpDir, "assets"),
      sourceDir: path.resolve(buildDirectory, "assets"),
      limit: 0,
      // extract all files
      fileName: "[dirname]/[name]-[hash][extname]"
    })]
  };
}
function defaultOutputOptions(buildDirectory) {
  return {
    format: "es",
    plugins: [rollupPluginTerser.terser()],
    manualChunks: id => {
      if (id.includes("web_modules")) {
        return path.parse(id).name;
      }
    },
    assetFileNames: "css/[name]-[hash].[ext]",
    chunkFileNames: "chunks/[name]-[hash].chunk.js",
    compact: true,
    sourcemap: true,
    entryFileNames: "entrypoints/[name]-[hash].js",
    dir: buildDirectory
  };
}

function parseHashFileName(filePath) {
  const {
    dir,
    base
  } = path.parse(filePath);
  const fileWithoutHash = base.replace(/(.*)-\w+(\.\w+)/g, "$1$2");
  return path.join(dir, fileWithoutHash);
}
function shellRun(cmd, options = {}) {
  options.stdio = options.stdio || "inherit";
  options.shell = options.shell || true;
  options.encoding = options.encoding || "utf8";
  return child_process.spawnSync(cmd, options);
}

// https://github.com/pikapkg/snowpack/blob/master/plugins/plugin-webpack/plugins/proxy-import-resolve.js
function proxyImportResolver(source) {
  const regex = /import.*['"].*\.(\w+)\.proxy\.js['"]/g;
  return source.replace(regex, (fullMatch, originalExt) => {
    // no JSON plugin loaded
    if (originalExt === "json") {
      return fullMatch;
    }

    return fullMatch.replace(".proxy.js", "");
  });
}

function addToManifest({
  manifest,
  chunkOrAsset,
  assignTo,
  useFileType = true
}) {
  const {
    fileName
  } = chunkOrAsset;
  const asset = chunkOrAsset;
  const assignment = assignTo;
  manifest[parseHashFileName(fileName)] = prependSlash(fileName);
  manifest[assignment] = manifest[assignment] || {};
  assignAsset({
    obj: manifest[assignment],
    asset,
    useFileType
  });
}
function prependSlash(fileName) {
  if (fileName[0] === "/") {
    return fileName;
  }

  return "/" + fileName;
}

function assignAsset({
  obj,
  asset,
  useFileType
}) {
  const {
    map,
    fileName
  } = asset;
  let fileType = extType(fileName);
  let baseName; // Non js / js.map / css / css.map files retain their extension

  if (fileType === null) {
    baseName = parseHashFileName(fileName);
  } else {
    baseName = path.parse(parseHashFileName(fileName)).name.split(".")[0];
  }

  const adjustedFileName = prependSlash(fileName);
  obj[baseName] = obj[baseName] || {};

  if (useFileType === false) {
    obj[baseName] = adjustedFileName;
    return;
  }

  obj[baseName][fileType] = adjustedFileName;

  if (map) {
    const mapFile = adjustedFileName + ".map";
    fileType = extType(mapFile);
    obj[baseName][fileType] = mapFile;
  }
}

function extType(fileName) {
  if (fileName.endsWith(".css.map")) {
    return "css.map";
  } else if (fileName.endsWith(".css")) {
    return "css";
  } else if (fileName.endsWith(".js.map")) {
    return "js.map";
  } else if (fileName.endsWith(".js")) {
    return "js";
  }

  return null;
}

/**
 * An instance of JSDOM
 * @typedef {string} JSDOM
 */

/**
 * A manifest file parsed to an Object
 * @typedef {Object} ManifestObject
 */

/**
 * Rewrites the values of scripts and extrapolates the css file
 *   associated with the script.
 *
 * @param {Object} params
 * @param {string} params.file - Path to a file
 * @param {ManifestObject} params.manifest - Manifest Object
 * @param {string} params.destFile - Path to the new file
 * @returns {undefined}
 */

function emitHtmlFiles({
  file,
  manifest,
  destFile,
  baseUrl
}) {
  const fileContents = fs.readFileSync(file, {
    encoding: "utf8"
  });
  const dom = new jsdom.JSDOM(fileContents, {
    includeNodeLocations: true
  });
  const newDom = rewriteScripts({
    dom,
    manifest,
    baseUrl
  });
  fs.writeFileSync(destFile, newDom.serialize(), "utf8");
}
/**
 * Rewrites the scripts of a given JSDOM instance to use files from the manifest
 * @param {Object} params
 * @param {JSDOM} params.dom - An instance of JSDOM
 * @param {ManifestObject} params.manifest - Manifest file parsed to Object
 * @returns {string} Returns a string from serializing JSDOM
 */

function rewriteScripts({
  dom,
  manifest,
  baseUrl
}) {
  const domDocument = dom.window.document;
  const scripts = domDocument.querySelectorAll("script");
  const unhashedEntrypoints = Object.keys(manifest.entrypoints).map(fileName => {
    return parseHashFileName(manifest.entrypoints[fileName]["js"]);
  });
  scripts.forEach(script => {
    if (!isEntrypoint({
      entrypoints: unhashedEntrypoints,
      script
    })) {
      return;
    }

    const baseFile = path.parse(script.src).name;
    const jsFile = manifest.entrypoints[baseFile].js;
    script.src = fixUrl({
      baseUrl,
      file: jsFile
    });
    const cssFile = manifest.entrypoints[baseFile].css;
    const stylesheet = domDocument.createElement("link");
    stylesheet.rel = "stylesheet";
    stylesheet.href = fixUrl({
      baseUrl,
      file: cssFile
    });
    insertBefore(script, stylesheet);
  });
  return dom;
}

function fixUrl({
  baseUrl,
  file
}) {
  baseUrl = baseUrl || "/";
  return url$1.parse(baseUrl).protocol ? url$1.resolve(baseUrl, file) : path.posix.join(baseUrl, file);
}

function insertBefore(existingNode, newNode) {
  existingNode.parentNode.insertBefore(newNode, existingNode);
}

function isEntrypoint({
  entrypoints,
  script
}) {
  // Remove trailing slashes
  script.src.replace(/\/$/, "");

  if (entrypoints.includes(script.src)) {
    return true;
  } // Account for src="entrypoints/blah"


  if (entrypoints.includes("/" + script.src)) {
    script.src = "/" + script.src;
    return true;
  }

  return false;
}

const TMP_BUILD_DIRECTORY = path.join(os.tmpdir(), "build");

function getEntrypoints(entrypoints) {
  if (typeof entrypoints === "string") {
    return glob.sync(entrypoints);
  }

  return entrypoints;
}

async function rollupBuild({
  snowpackConfig,
  pluginOptions,
  inputOptions,
  outputOptions
}) {
  const baseUrl = snowpackConfig.devOptions.baseUrl || "/";
  const TMP_DEBUG_DIRECTORY = path.join(os.tmpdir(), "_source_");
  const entrypoints = getEntrypoints(pluginOptions.entrypoints);
  inputOptions.input = inputOptions.input || entrypoints;
  const buildDirectory = outputOptions.dir;
  outputOptions.dir = TMP_BUILD_DIRECTORY;
  const bundle = await rollup.rollup(inputOptions);
  const {
    output
  } = await bundle.generate(outputOptions);
  const manifest = {};

  for (const chunkOrAsset of output) {
    if (chunkOrAsset.isEntry || chunkOrAsset.type === "asset") {
      addToManifest({
        manifest,
        chunkOrAsset,
        assignTo: "entrypoints"
      });
      continue;
    }

    addToManifest({
      manifest,
      chunkOrAsset,
      assignTo: "chunks"
    });
  }

  await bundle.write(outputOptions); // Add assets to manifest, use path.relative to fix minor issues

  glob.sync(`${TMP_BUILD_DIRECTORY}/**/*.*`).forEach(fileName => {
    fileName = path.relative(TMP_BUILD_DIRECTORY, fileName);
    const chunkOrAsset = {
      fileName,
      map: null
    };
    addToManifest({
      manifest,
      chunkOrAsset,
      assignTo: "assets",
      useFileType: false
    });
  });
  const manifestJSON = JSON.stringify(manifest, null, 2);
  fs.writeFileSync(path.join(TMP_BUILD_DIRECTORY, "manifest.json"), manifestJSON);

  if (pluginOptions.emitHtml === true) {
    glob.sync(buildDirectory + "**/*.html").forEach(file => {
      let destFile = path.relative(buildDirectory, file);
      destFile = path.join(TMP_BUILD_DIRECTORY, destFile);
      emitHtmlFiles({
        file,
        manifest,
        destFile,
        baseUrl
      });
    });
  }

  shellRun(`rm -rf ${TMP_DEBUG_DIRECTORY} && mkdir -p ${TMP_DEBUG_DIRECTORY}`);
  shellRun(`mv ${buildDirectory} ${TMP_DEBUG_DIRECTORY}`);
  shellRun(`mv ${TMP_BUILD_DIRECTORY} ${buildDirectory}`);

  if (pluginOptions.preserveSourceFiles === true) {
    const buildDebugDir = path.join(buildDirectory, "_source_");
    shellRun(`mv ${TMP_DEBUG_DIRECTORY}/ ${buildDebugDir}`);
  }
}

const plugin = (snowpackConfig, pluginOptions = {}) => {
  snowpackConfig.buildOptions.minify = false; // Let rollup handle this

  snowpackConfig.buildOptions.clean = true;
  return {
    name: "snowpack-plugin-rollup-bundle",

    async optimize({
      buildDirectory
    }) {
      const inputOptions = defaultInputOptions({
        buildDirectory,
        tmpDir: TMP_BUILD_DIRECTORY
      });
      const outputOptions = defaultOutputOptions(buildDirectory);

      let extendConfig = cfg => cfg;

      if (typeof pluginOptions.extendConfig === "function") {
        extendConfig = pluginOptions.extendConfig;
      } else if (typeof pluginOptions.extendConfig === "object") {
        extendConfig = cfg => _objectSpread2(_objectSpread2({}, cfg), pluginOptions.extendConfig);
      }

      const extendedConfig = await extendConfig({
        snowpackConfig: _objectSpread2({}, snowpackConfig),
        pluginOptions: _objectSpread2({}, pluginOptions),
        inputOptions: _objectSpread2({}, inputOptions),
        outputOptions: _objectSpread2({}, outputOptions)
      }); // Rewrite "proxy.js" imports prior to building

      glob.sync(buildDirectory + "/**/*.js").forEach(file => {
        const resolvedImports = proxyImportResolver(fs.readFileSync(file, "utf8"));
        fs.writeFileSync(file, resolvedImports, "utf8");
      });
      await rollupBuild(extendedConfig);
    }

  };
};

exports.default = plugin;
//# sourceMappingURL=index.js.map
